/*
 * Created by CSD YACC from "pfe2mf.y"
 */

# line 2
#include <stdio.h>
#define END(v)  (v-1 + sizeof v / sizeof v[0])
extern int yylineno;

/* Pfe2mf - syntax analysis
 *
 * Copyright (c) 1992 by Warren Toomey.
 *
 * You may distribute this code for free (or a nominal copying fee) if you
 * do not alter the source code at all. If you want to distribute a changed
 * version, you should distribute a set of patches along with the original.
 *
 * You may not sell this source code or profit in any way from this code.
 */

/* Every stroke in a path consists of a node of this type.
 * The id indicates what stroke type it is. The closepath
 * field is TRUE for the last stroke in a path.
 * Id's of type LINE use the start & end fields. Id's of
 * type CURVE use the start, end and bc [Bezier control]
 * fields.
 */
struct pathstruct
	{
	  int startx;
	  int starty;
	  int endx;
	  int endy;
	  int bc1x;
	  int bc1y;
	  int bc2x;
	  int bc2y;
	  int id;
	  int closepath;
	  struct pathstruct *next;
	};
#define LINE	1
#define CURVE	2

/* Global variables.
 */
int X=0,Y=0;			/* Current X,Y position */
int Unknown_chcode=128;		/* Used when we can't find a TeX equivalent */
char CHname[100];		/* Character's name */
int dummyi, dummyj;		/* Dummy variables */
int Char_code;			/* This character's char code */
int Leftgap;			/* Left gap as given from `hsbw' command */
int Width;			/* Character's width */
int Height;			/* Character's height */
int Depth;			/* Character's depth */
int Space;			/* Inter-word space */
int Stretch;			/* Inter-word stretch */
int Shrink;			/* Inter-word shrink */
int Fi=0, Fl=0, Ff=0;		/* True if we've got these characters */
int Ffi=0, Ffl=0;		/* True if we've got these characters */
int Howfill;			/* Hint as how to fill the character */
struct pathstruct *Head,
		  *Tail,
		  *This;	/* Start, end, current node of stroke list */
char *Pfefile=NULL;		/* Name of pfefile */
int Allchars=0;			/* If 1, we parse all chars. Otherwise, just */
				/* the ones with TeX equivalents. */
char *FontName=NULL;		/* The name of the font */

# line 67

#define UNION 1
typedef union  {
	 int value;
	 char *name;
	} YYSTYPE;
# define IDENTIFIER 257
# define NUMBER 258
# define CALLSUBR 259
# define CLOSEPATH 260
# define ENDFILE 261
# define ENDCHAR 262
# define HLINETO 263
# define HMOVETO 264
# define HSBW 265
# define HSTEM 266
# define HVCURVETO 267
# define RLINETO 268
# define RMOVETO 269
# define RRCURVETO 270
# define SEAC 271
# define VHCURVETO 272
# define VLINETO 273
# define VMOVETO 274
# define VSTEM 275
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
extern YYSTYPE yylval;  /*CSD & DECUS LEX */
YYSTYPE yyval;          /*CSD & DECUS LEX */
# define YYERRCODE 256

#line 247

/* Get a line of input into the buffer.
 * Do not exceed the buffer size. Return
 * the number of chars in the buffer.
 * Ignore leading whitespace.
 */
int getline(buf,size)
 char *buf;
 int size;
 {
  int ch;
  int numgot=0;

  size--;
  while ((ch=getchar())!=EOF && (ch==' ' || ch=='\t' || ch=='\n'))
   if (ch=='\n') yylineno++;

  while(ch!=EOF && ch!='\n' && numgot<size)
   {
    buf[numgot++]= ch; ch=getchar();
   }
  buf[numgot++]='\0'; yylineno++;
  return(numgot);
 }

void usage()
 { fprintf(stderr,"Usage: pfe2mf [-a] pfefile\n"); exit(1); 
 }


/* Wade thru the first half of the pfe file
 *
 */
parsepfe()
 {
  char inbuf[500];			/* Should be enough! */
  int i,amt;
  int percentleft=1;

  printf("%% Information from the pfe file:\n%%\n");

  while(1)
   {
    amt= getline(inbuf,500);
    if (amt==0) exit(1);
    if (percentleft && inbuf[0]=='%') puts(inbuf);
    else percentleft=0;
    if (strstr(inbuf,"CharString")) return;	/* End of top half */
    if (!strncmp(inbuf,"/FontName",9) && FontName==NULL)
     {
      FontName= (char *)malloc(strlen(inbuf)-9);
      if (FontName)
	for (i=0,amt=11; inbuf[amt]!=' ' && inbuf[amt]!='\0';)
		FontName[i++]= inbuf[amt++];
	FontName[i]='\0';
      continue;
     }
    if (!strncmp(inbuf,"/version",8))
	{ inbuf[0]='%'; puts(inbuf); continue; }
    if (!strncmp(inbuf,"/Notice",7))
	{ inbuf[0]='%'; puts(inbuf); continue; }
    if (!strncmp(inbuf,"/FullName",9))
	{ inbuf[0]='%'; puts(inbuf); continue; }
    if (!strncmp(inbuf,"/FamilyName",11))
	{ inbuf[0]='%'; puts(inbuf); continue; }
    if (!strncmp(inbuf,"/Weight",7))
	{ inbuf[0]='%'; puts(inbuf); continue; }
   }
 }

/* Print out the header of the Metafont Code.
 */
void printheader()
 {
  
  printf("%%\n%%\n%% The following constants convert Adobe Type 1 binary\n");
  printf("%% point size into Metafont values. You may\n");
  printf("%% want to alter these values to get the\n");
  printf("%% correct point size output by Metafont\n%%\n%%\n");

/* I can't seem to get Metafont to produce the
 * same charwidths as afm2tfm. I have tried real numbers
 * as well as integers. 100 gives the closest value.
 */

  printf("ap#:= pt#/100;\nap= pt/100;\n\n");

  if (FontName) printf("font_identifier \"%s\";\n",FontName);
  printf("font_size 10pt#;\n\n\n");
 }


static struct chrtable {	/* character lookup table */
	char *ch_name;		/* character's Adobe name */
	int ch_code;		/* equivalent TeX code */
	int how_fill;		/* plus how to fill hint */
    } chtable[]=		/* sorted */
       {
	".notdef", -1, 1,
	"A", 65, 0,
	"AE", 29, 0,
	"B", 66, 0,
	"C", 67, 1,
	"D", 68, 0,
	"E", 69, 1,
	"F", 70, 1,
	"G", 71, 1,
	"H", 72, 1,
	"I", 73, 1,
	"J", 74, 1,
	"K", 75, 1,
	"L", 76, 1,
	"M", 77, 1,
	"N", 78, 1,
	"O", 79, 0,
	"OE", 30, 0,
	"Oslash", 31, 0,
	"P", 80, 0,
	"Q", 81, 0,
	"R", 82, 0,
	"S", 83, 1,
	"T", 84, 1,
	"U", 85, 1,
	"V", 86, 1,
	"W", 87, 1,
	"X", 88, 1,
	"Y", 89, 1,
	"Z", 90, 1,
	"a", 97, 0,
	"acute", 19, 1,
	"ae", 26, 0,
	"ampersand", 38, 0,
	"asterisk", 42, 1,
	"at", 64, 0,
	"b", 98, 0,
	"bracketleft", 91, 1,
	"bracketright", 93, 1,
	"breve", 21, 1,
	"c", 99, 1,
	"caron", 20, 1,
	"cedilla", 24, 1,
	"circumflex", 94, 1,
	"colon", 58, 1,
	"comma", 44, 1,
	"d", 100, 0,
	"dieresis", 127, 1,
	"dollar", 36, 0,
	"dotaccent", 95, 1,
	"e", 101, 0,
	"eight", 56, 0,
	"emdash", 124, 1,
	"endash", 123, 1,
	"equal", 61, 1,
	"exclam", 33, 1,
	"f", 102, 1,
	"ff", 11, 1,
	"ffi", 14, 1,
	"ffl", 15, 1,
	"fi", 12, 1,
	"five", 53, 1,
	"fl", 13, 1,
	"four", 52, 0,
	"g", 103, 0,
	"germandbls", 25, 1,
	"grave", 18, 1,
	"greater", 62, 1,
	"h", 104, 1,
	"hyphen", 45, 1,
	"i", 105, 1,
	"j", 106, 1,
	"k", 107, 1,
	"l", 108, 1,
	"less", 60, 1,
	"m", 109, 1,
	"macron", 22, 1,
	"n", 110, 1,
	"nine", 57, 0,
	"numbersign", 35, 0,
	"o", 111, 0,
	"oe", 27, 0,
	"one", 49, 1,
	"oslash", 28, 0,
	"p", 112, 0,
	"parenleft", 40, 1,
	"parenright", 41, 1,
	"percent", 37, 1,
	"period", 46, 1,
	"plus", 43, 1,
	"q", 113, 0,
	"question", 63, 1,
	"quotedbl", 125, 1,
	"quotedblleft", 92, 1,
	"quotedblright", 34, 1,
	"quoteleft", 96, 1,
	"quoteright", 39, 1,
	"r", 114, 1,
	"ring", 23, 0,
	"s", 115, 1,
	"semicolon", 59, 1,
	"seven", 55, 1,
	"six", 54, 0,
	"slash", 47, 1,
	"t", 116, 1,
	"three", 51, 1,
	"tilde", 126, 1,
	"two", 50, 1,
	"u", 117, 1,
	"v", 118, 1,
	"w", 119, 1,
	"x", 120, 1,
	"y", 121, 1,
	"z", 122, 1,
	"zero", 48, 0,
    };

int chscreen(name)	/* Get charcode from Identifier */
 char *name;
 {
  struct chrtable *low = chtable,
		 *high = END(chtable),
		 *mid;
  int c;

  name++;			/* Skip leading slash */
  strcpy(CHname, name);

  while (low<=high)
   {
    mid= low + (high-low)/2;
    if ((c = strcmp(mid->ch_name, name))==0)
	{ Char_code= mid->ch_code;
	  Howfill= mid->how_fill;
	  return;
	}
    else if (c<0)
	low= mid + 1;
    else
	high= mid - 1;
   }
#ifdef DEBUG
  fprintf(stderr,"Unknown charname %s\n", name);
#endif
  Char_code= Unknown_chcode++;
  if (Unknown_chcode==256) Unknown_chcode--;
  Howfill=0;
  return;
 }

/* Create a new node pointed at by This. Add This to the
 * tail of the Pathlist
 */
void newnode()
 {
  This= (struct pathstruct *)malloc(sizeof(struct pathstruct));
  if (This==NULL) { fprintf(stderr,"Malloc failed in  newnode\n"); exit(1); }

  if (Head==NULL) Head=Tail=This;
  else { Tail->next= This; Tail= This; }
  This->startx= X;
  This->starty= Y;
  This->closepath= 0;
  This->next= NULL;
 }

/* Deduce the height and depth of a character based
 * on the maximum y co-ordinates in the paths.
 * We add on 3% to each for good measure.
 *
 * While we're here, we calculate the interword space, stretch
 * and shrink, using the widths for x, M, and i respectively.
 */
void gethtdepth(chcode)
 int chcode;
 {
  Height=0; Depth=0;

  for (This= Head; This; This=This->next)
   {
    if (This->starty > Height) Height= This->starty;
    if (This->endy > Height) Height= This->endy;
    if (This->starty < Depth) Depth= This->starty;
    if (This->endy < Depth) Depth= This->endy;
   }

  Height+= Height * 3 / 100;
  Depth= -Depth; Depth+= Depth * 3 / 100;
 
  switch(chcode)
   {
	case 120: Space= Width;			/* x */
		  break;
	case 77: Stretch= Width;		/* M */
		  break;
	case 105: Shrink= Width;		/* i */
   }
 }


/* Free a character - don't print out the code
 */
int freecharacter()
 {
  for (This= Head; This;)
   {
    Tail= This; This=This->next; free(Tail);
   }
 }


/* Print out the Metafont code for the
 * character.
 */
int givecharacter()
 {
  int startnode=1;

  printf("beginchar(%d, ",Char_code);
  gethtdepth(Char_code);		/* Have we got Height & Depth? */

  switch(Char_code)			/* Check for the ff ligatures */
   {
	case 11:  Ff= 1;
		  break;
	case 12:  Fi= 1;
		  break;
	case 13:  Fl= 1;
		  break;
	case 14:  Ffi= 1;
		  break;
	case 15:  Ffl= 1;
   }

  printf(" %dap#, %dap#, %dap#); ",Width, Height, Depth);
  printf("\"The letter %s\";\n", CHname);

  for (This= Head; This;)
   {
/* The following is a heuristic to determine if we
 * should fill or unfill. We fill for the first
 * stroke and unfill the others, unless Howfill is 1.
 * This should be replaced with an algorithm for
 * determining if a path is clockwise or not.
 */

    if (startnode)			/* We must issue a fill command */
      { startnode=0;
	if (This == Head || Howfill) printf("fill\n");
	else printf("unfill\n");
      }

    switch(This->id)
     {
      case LINE:
		printf(" (%dap,%dap) -- ",This->startx,This->starty);
		printf("(%dap,%dap) &\n",This->endx,This->endy);
		break;
      case CURVE:
		printf(" (%dap,%dap) .. ",This->startx,This->starty);
		printf("controls (%dap,%dap) ",This->bc1x,This->bc1y);
		printf("and (%dap,%dap) ",This->bc2x,This->bc2y);
		printf(".. (%dap,%dap) &\n",This->endx,This->endy);
		break;
      default:
		fprintf(stderr,"Internal error - bad pathnode id %d\n",
			This->id);
		exit(1);
     }
    if (This->closepath) { printf("cycle;\n"); startnode=1; }

    Tail= This; This=This->next; free(Tail);
   }
  printf("endchar;\n");
 }

/* Print out any ligature table if we have any ff chars
 */
int giveligtable()
 {
  int i,j;

  i= Fi + Ff + Fl;
  j= Ffi + Ffl;
  if (i)				/* We can make a ligtable */
   {
    printf("\nligtable \"f\": ");

    if (Fi)
      { printf("\"i\"=:12");
        if (--i) printf(", "); else printf(";\n");
      }
    if (Ff)
      { printf("\"f\"=:11");
    	if (--i) printf(", "); else printf(";\n");
      }
    if (Fl)
      { printf("\"l\"=:13");
    	if (--i) printf(", "); else printf(";\n");
      }

    if (j)
     {
      printf("ligtable 11: ");
      if (Ffi)
	{ printf("\"i\"=:14");
      	  if (--j) printf(", "); else printf(";\n");
        }
      if (Ffl)
	{ printf("\"l\"=:15");
	  if (--j) printf(", "); else printf(";\n");
        }
     }
   }
 }

int main(argc, argv)
 int argc;
 char *argv[];
 {
  int i;

  switch(argc)
   {
    case 2:	Pfefile= argv[1];
		break;
    case 3:	if (!strcmp("-a",argv[1])) Allchars= 1;
		else usage();
		Pfefile= argv[2];
		break;
    default:	usage();
   }

  if (freopen(Pfefile, "r", stdin)==NULL)
    { fprintf(stderr, "Couldn't open %s\n", Pfefile); exit(1); }
   
#ifdef OLDFLEX
/* Other lexers may need this too :-( */

  setbuf(stdin, NULL);
#endif
  printf("%%%% Metafont code produced by pfe2mf\n%%%% from file");
  printf(" %s.\n%%\n",Pfefile);

  parsepfe();
  printheader();
  i= yyparse();
#ifdef DEBUG
  fprintf(stderr,"yyparse() == %d\n",i);
#endif
 }
short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 32
# define YYLAST 88
short yyact[]={

  43,  42,  52,  35,  38,   5,  26,  50,  41,   6,
  37,  39,  41,  49,  37,  39,  27,  40,  36,  31,
  51,  40,  46,  26,  29,  28,  31,  47,  48,  43,
  26,  29,  50,  27,  30,  45,  34,  46,  32,  37,
  27,  30,  47,  25,  14,  19,  19,  23,  11,  44,
  22,   9,   5,  18,  17,   7,   3,  16,  15,   7,
   8,  13,  21,  20,  21,  20,  18,  17,  18,  17,
  16,  12,  16,   4,  33,  10,   8,   2,   4,   1,
  15,   0,  15,   0,  13,  12,  24,  10 };
short yypact[]={

-205,-1000,-252,-1000,-207,-1000,-1000,-1000,-214,-208,
-215,-1000,-1000,-1000,-233,-1000,-1000,-1000,-1000,-1000,
-222,-1000,-262,-1000,-1000,-240,-1000,-1000,-254,-1000,
-1000,-1000,-1000,-1000,-257,-1000,-258,-1000,-209,-1000,
-1000,-1000,-229,-223,-230,-245,-1000,-1000,-251,-226,
-268,-1000,-1000 };
short yypgo[]={

   0,  79,  77,  56,  73,  55,  78,  76,  75,  71,
  61,  58,  57,  54,  53 };
short yyr1[]={

   0,   1,   2,   2,   3,   3,   3,   6,   6,   8,
   8,   8,   8,  11,  11,  13,  13,  14,  14,  14,
  14,  14,  14,  10,  10,  10,   9,   9,  12,   5,
   7,   4 };
short yyr2[]={

   0,   2,   1,   2,   4,   3,   3,   1,   2,   1,
   1,   1,   1,   1,   2,   1,   2,   2,   2,   3,
   5,   5,   7,   2,   2,   3,   3,   3,   2,   3,
   6,   1 };
short yychk[]={

-1000,  -1,  -2,  -3,  -4, 257, 261,  -3,  -5, 258,
  -6, 262,  -7,  -8, 258,  -9, -10, -11, -12, 260,
 -13, -14, 258, 262,  -8, 258, 263, 273, 258, 264,
 274, 259, 260, -14, 258, 265, 258, 268, 258, 269,
 275, 266, 258, 258, 258, 258, 267, 272, 258, 258,
 258, 271, 270 };
short yydef[]={

   0,  -2,   0,   2,   0,  31,   1,   3,   0,   0,
   0,   5,   6,   7,   0,   9,  10,  11,  12,  13,
   0,  15,   0,   4,   8,   0,  17,  18,   0,  23,
  24,  28,  14,  16,   0,  29,   0,  19,   0,  25,
  26,  27,   0,   0,   0,   0,  20,  21,   0,   0,
   0,  30,  22 };
# define YYFLAG -1000
# define YYERROR goto yyerrlab
# define YYACCEPT return(0)
# define YYABORT return(1)

	/* parser for yacc output */

#ifdef YYDEBUG
int yydebug = 0;		/* 1 for debugging */
#endif

YYSTYPE yylval;			/* where should this be?? */
YYSTYPE yyv[YYMAXDEPTH];	/* where the values are stored */
int yychar = -1;		/* current input token number */
int yynerrs = 0;		/* number of errors */
short yyerrflag = 0;		/* error recovery flag */

yyparse() {

	short yys[YYMAXDEPTH];
	short yyj, yym;
	register YYSTYPE *yypvt;
	register short yystate, *yyps, yyn;
	register YYSTYPE *yypv;
	register short *yyxi;

	yystate = 0;
	yychar = -1;
	yynerrs = 0;
	yyerrflag = 0;
	yyps= &yys[-1];
	yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

#ifdef YYDEBUG
	if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
#endif
		if( ++yyps>= &yys[YYMAXDEPTH] )
		    { yyerror( "yacc stack overflow" ); return(1); }
		*yyps = yystate;
		++yypv;
		*yypv = yyval;

 yynewstate:

	yyn = yypact[yystate];

	if( yyn<= YYFLAG ) goto yydefault; /* simple state */

	if( yychar<0 ) if( (yychar=yylex())<0 ) yychar=0;
	if( (yyn += yychar)<0 || yyn >= YYLAST ) goto yydefault;

	if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
		yychar = -1;
		yyval = yylval;
		yystate = yyn;
		if( yyerrflag > 0 ) --yyerrflag;
		goto yystack;
		}

 yydefault:
	/* default state action */

	if( (yyn=yydef[yystate]) == -2 ) {
		if( yychar<0 ) if( (yychar=yylex())<0 ) yychar = 0;
		/* look through exception table */

		for( yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=yystate) ; yyxi += 2 ) ; /* VOID */

		while( *(yyxi+=2) >= 0 ){
			if( *yyxi == yychar ) break;
			}
		if( (yyn = yyxi[1]) < 0 ) return(0);   /* accept */
		}

	if( yyn == 0 ){ /* error */
		/* error ... attempt to resume parsing */

		switch( yyerrflag ){

		case 0:   /* brand new error */

			yyerror( "syntax error" );
		yyerrlab:
			++yynerrs;

		case 1:
		case 2: /* incompletely recovered error ... try again */

			yyerrflag = 3;

			/* find a state where "error" is a legal shift action */

			while ( yyps >= yys ) {
			   yyn = yypact[*yyps] + YYERRCODE;
			   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
			      yystate = yyact[yyn];  /* simulate a shift of "error" */
			      goto yystack;
			      }
			   yyn = yypact[*yyps];

			   /* the current yyps has no shift onn "error", pop stack */

#ifdef YYDEBUG
			   if( yydebug ) printf( "error recovery pops state %d, uncovers %d\n", *yyps, yyps[-1] );
#endif
			   --yyps;
			   --yypv;
			   }

			/* there is no state on the stack with an error shift ... abort */

	yyabort:
			return(1);


		case 3:  /* no shift yet; clobber input char */

#ifdef YYDEBUG
			if( yydebug ) printf( "error recovery discards char %d\n", yychar );
#endif

			if( yychar == 0 ) goto yyabort; /* don't discard EOF, quit */
			yychar = -1;
			goto yynewstate;   /* try again in the same state */

			}

		}

	/* reduction by production yyn */

#ifdef YYDEBUG
		if( yydebug ) printf("reduce %d\n",yyn);
#endif
		yyps -= yyr2[yyn];
		yypvt = yypv;
		yypv -= yyr2[yyn];
		yyval = yypv[1];
		yym=yyn;
			/* consult goto table to find next state */
		yyn = yyr1[yyn];
		yyj = yypgo[yyn] + *yyps + 1;
		if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn ) yystate = yyact[yypgo[yyn]];
		switch(yym){
			
case 1:
# line 96
{ 			/* Finally finished!! */
		  giveligtable();	/* Print any ligtable */
					/* Write out the inter */
					/* word space stuff */
		  printf("\nfont_normal_space %dap#;\n", Space);
		  printf("font_normal_stretch %dap#;\n", Stretch);
		  printf("font_normal_space %dap#;\n", Shrink);
		  printf("\n\n\n%% End of output from pfe2mf\n");
		  exit(0);
		} break;
case 4:
# line 115
{ if (Allchars || (Char_code>=0 && Char_code<128))
			givecharacter();
		  else freecharacter();
		 } break;
case 5:
# line 120
{ if (Allchars || (Char_code>=0 && Char_code<128))
			givecharacter();
		  else freecharacter();
		 } break;
case 13:
# line 141
{ if (This) This->closepath= 1; } break;
case 14:
# line 143
{ if (This) This->closepath= 1; } break;
case 17:
# line 153
{
		  newnode();
		  X+= yypvt[-1].value;
		  This->endx= X; This->endy= Y;
		  This->id= LINE;
		} break;
case 18:
# line 160
{
		  newnode();
		  Y+= yypvt[-1].value;
		  This->endx= X; This->endy= Y;
		  This->id= LINE;
		} break;
case 19:
# line 167
{
		  newnode();
		  X+= yypvt[-2].value; Y+= yypvt[-1].value;
		  This->endx= X; This->endy= Y;
		  This->id= LINE;
		} break;
case 20:
# line 174
{
		  newnode();
		  X+= yypvt[-4].value;
		  This->bc1x= X; This->bc1y= Y;
		  X+= yypvt[-3].value; Y+= yypvt[-2].value;
		  This->bc2x= X; This->bc2y= Y;
		  Y+= yypvt[-1].value;
		  This->endx= X; This->endy= Y;
		  This->id= CURVE;
		} break;
case 21:
# line 185
{
		  newnode();
		  Y+= yypvt[-4].value;
		  This->bc1x= X; This->bc1y= Y;
		  X+= yypvt[-3].value; Y+= yypvt[-2].value;
		  This->bc2x= X; This->bc2y= Y;
		  X+= yypvt[-1].value;
		  This->endx= X; This->endy= Y;
		  This->id= CURVE;
		} break;
case 22:
# line 196
{
		  newnode();
		  X+= yypvt[-6].value; Y+= yypvt[-5].value;
		  This->bc1x= X; This->bc1y= Y;
		  X+= yypvt[-4].value; Y+= yypvt[-3].value;
		  This->bc2x= X; This->bc2y= Y;
		  X+= yypvt[-2].value; Y+= yypvt[-1].value;
		  This->endx= X; This->endy= Y;
		  This->id= CURVE;
		} break;
case 23:
# line 211
{ X+= yypvt[-1].value; } break;
case 24:
# line 213
{ Y+= yypvt[-1].value; } break;
case 25:
# line 215
{ X+= yypvt[-2].value; Y+= yypvt[-1].value; } break;
case 28:
# line 225
{ fprintf(stderr, "pfe2mf doesn't handle subroutines yet\n");
		} break;
case 29:
# line 231
{
		  Leftgap= yypvt[-2].value; Width= yypvt[-1].value;
		  X=yypvt[-2].value; Y=0;
		} break;
case 31:
# line 243
{ chscreen(yypvt[-0].name);
		  Head= Tail= NULL;
		} break;/* End of actions */
		}
		goto yystack;  /* stack new state and value */

	}
